package org.openxma.dsl.platform.dao.impl;

import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.criterion.Conjunction;
import org.hibernate.criterion.CriteriaSpecification;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.SimpleExpression;
import org.openxma.dsl.platform.dao.FilterExpression;
import org.openxma.dsl.platform.dao.Filterable;
import org.openxma.dsl.platform.dao.impl.ComparisonExpression.EqualsExpression;
import org.openxma.dsl.platform.dao.impl.ComparisonExpression.GreaterThanExpression;
import org.openxma.dsl.platform.dao.impl.ComparisonExpression.GreaterThanOrEqualsExpression;
import org.openxma.dsl.platform.dao.impl.ComparisonExpression.LessThanExpression;
import org.openxma.dsl.platform.dao.impl.ComparisonExpression.LessThanOrEqualsExpression;
import org.openxma.dsl.platform.dao.impl.ComparisonExpression.LikeExpression;
import org.openxma.dsl.platform.dao.impl.CompositeExpression.AndExpression;
import org.openxma.dsl.platform.dao.impl.CompositeExpression.OrExpression;
import org.openxma.dsl.platform.dao.impl.UnaryExpression.NotExpression;

/**
 * first draft of a filterable HibernateFilter
 */
public class HibernateFilter implements Filterable {

	private Criteria criteria;
	private String alias;

	public HibernateFilter(Criteria criteria) {
		this(criteria,"");
	}

	public HibernateFilter(Criteria criteria, String alias) {
		super();
		this.criteria = criteria;
		this.alias = alias;
	}

	public Object append(EqualsExpression equalsExpression) {
		String propertyName = (String) equalsExpression.getLeft().applyTo(this);
		Object propertyValue = equalsExpression.getRight().applyTo(this);
		SimpleExpression eq = Restrictions.eq(getPropertyPath(propertyName), propertyValue);
		if (equalsExpression.isIgnoreCase()) {
			eq = eq.ignoreCase();
		}
		return eq;
	}

	public Object append(LikeExpression likeExpression) {
		String propertyName = (String) likeExpression.getLeft().applyTo(this);
		Object propertyValue = likeExpression.getRight().applyTo(this);
		SimpleExpression like = Restrictions.like(getPropertyPath(propertyName), propertyValue);
		if (likeExpression.isIgnoreCase()) {
			like = like.ignoreCase();
		}
		return like;
	}

	public Object append(AndExpression andFilterExpression) {
		Conjunction conjunction = Restrictions.conjunction();
		for (FilterExpression filterExpression : andFilterExpression.getFilterExpressions()) {
			Object applyTo = filterExpression.applyTo(this);
			if (applyTo instanceof Criterion) {
				conjunction.add((Criterion) applyTo);
			}
		}
		return conjunction;
	}

	public Object append(OrExpression orFilterExpression) {
		Disjunction conjunction = Restrictions.disjunction();
		for (FilterExpression filterExpression : orFilterExpression.getFilterExpressions()) {
			conjunction.add((Criterion) filterExpression.applyTo(this));
		}
		return conjunction;
	}

	public Object append(ResultLimitExpression limitFilterExpression) {
		this.criteria.setFirstResult(limitFilterExpression.getFirstResult().intValue())
				.setMaxResults(limitFilterExpression.getMaxResults().intValue());
		return this.criteria;

	}

	public Object append(NotExpression notExpression) {
		return Restrictions.not((Criterion) notExpression.getFilterExpression().applyTo(this));
	}

	public Object append(JoinExpression joinExpression) {
		this.criteria = this.criteria.createAlias(getPropertyPath(joinExpression.getAlias()), joinExpression.getAlias(), 
				joinExpression .isLeftJoin() ? CriteriaSpecification.LEFT_JOIN : CriteriaSpecification.INNER_JOIN);
		this.criteria.setFetchMode(getPropertyPath(joinExpression.getAlias()), joinExpression.isFetch() ? FetchMode.JOIN : FetchMode.DEFAULT);
		FilterExpression filterExpression = joinExpression.getFilterExpression();
		return filterExpression.applyTo(new HibernateFilter(this.criteria,joinExpression.getAlias()));
	}
	
	public Object append(GreaterThanExpression greaterThanExpression) {
		String propertyName = (String) greaterThanExpression.getLeft().applyTo(this);
		Object propertyValue = greaterThanExpression.getRight().applyTo(this);
		return Restrictions.gt(getPropertyPath(propertyName), propertyValue);
	}
	
	public Object append(LessThanExpression lessThanExpression) {
		String propertyName = (String) lessThanExpression.getLeft().applyTo(this);
		Object propertyValue = lessThanExpression.getRight().applyTo(this);
		return Restrictions.lt(getPropertyPath(propertyName), propertyValue);
	}
	

	public Object append(GreaterThanOrEqualsExpression greaterThanOrEqualsExpression) {
		String propertyName = (String) greaterThanOrEqualsExpression.getLeft().applyTo(this);
		Object propertyValue = greaterThanOrEqualsExpression.getRight().applyTo(this);
		return Restrictions.ge(getPropertyPath(propertyName), propertyValue);
	}

	public Object append(LessThanOrEqualsExpression lessThanOrEqualsExpression) {
		String propertyName = (String) lessThanOrEqualsExpression.getLeft().applyTo(this);
		Object propertyValue = lessThanOrEqualsExpression.getRight().applyTo(this);
		return Restrictions.le(getPropertyPath(propertyName), propertyValue);
	}
	
	private String getPropertyPath(String propertyName) {
		if (null==this.alias || "".equals(this.alias)) {
			return propertyName;
		} else {
			return this.alias+"."+propertyName;
		}
	}

}
