package org.openxma.dsl.platform.service;


import java.util.Collection;
import java.util.List;

/**
 * <p>
 * Represents an object that sets up a communication between two independent
 * objects as defined in the Pattern of Enterprise Application Catalog. (see <a
 * href="http://martinfowler.com/eaaCatalog/mapper.html">Mapper</a>)
 * </p>
 * 
 * @version $Revision: 6664 $ $Date: 2010-09-29 18:47:40 +0200 (Wed, 29 Sep 2010) $
 */
public interface Mapper {
    /**
	 * @see #createAndMapOne(Object, Class, Object)
	 */
	<T> T createAndMapOne(Object sourceObject, Class<T> targetClass);

	/**
	 * creates and maps a new target object from a single source object
	 * 
	 * @param sourceObject  the source object to map
	 * @param targetClass   the target class
	 * @param context       the context used to differentiate between different mapping use cases
	 * @return an mapped instance of the given targetClass
	 */
	<T> T createAndMapOne(Object sourceObject, Class<T> targetClass, /*Map*/Object context);
	
    /**
     * @see #mapOne(Object, Object, Object)
     */
    <T> T mapOne(Object sourceObject, T targetObject);
    
    /**
     * maps from a single source object
     * 
     * @param sourceObject the source object to map
     * @param targetObject the target object
     * @param context      the context used to differentiate between different mapping use cases 
     * @return an mapped instance of the given targetObject
     */
    <T> T mapOne(Object sourceObject, T targetObject, /*Map*/Object context);

    /**
     * @see #createAndMapMany(Collection, Class, Object)
     */
    <T> List<T> createAndMapMany(Collection<?> sourceCollection, Class<T> targetClass);
    
    /**
     * creates a new target object and maps from a collection of source objects
     * 
     * @param sourceCollection the source collection to map
     * @param targetClass the target class
     * @param context       the context used to differentiate between different mapping use cases 
     * @return a list containing mapped instances of the given class
     */
    <T> List<T> createAndMapMany(Collection<?> sourceCollection, Class<T> targetClass, /*Map*/Object context);
    
}