package org.openxma.dsl.platform.xma.client.table;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import at.spardat.xma.mdl.table.ITableWMClient;
import at.spardat.xma.mdl.table.TableLayoutManager;

public class TableState {

    int[] originalColumnOrder;
    boolean[] stateValues;      
    boolean[] newStateValues;
    List<ColumnInfo> columStates;
    
    Table tableWidget;
    ITableWMClient tableModel;
    TableLayoutManager tableLayoutManager;      
    
    TableState(TableLayoutManager tableLayoutManager, Table tableWidget, ITableWMClient table, int[] originalColumnOrder) {
        this.originalColumnOrder = originalColumnOrder;
        columStates = new ArrayList<ColumnInfo>();
        for (int i=0;i<originalColumnOrder.length;i++) {                
            ColumnInfo columnState = new ColumnInfo();
            columnState.index = originalColumnOrder[i];
            columnState.minWidth = tableLayoutManager.getMinWidth(columnState.index);
            columnState.absoluteWidth = tableLayoutManager.getAbsolutWidth(columnState.index);
            columnState.percent = tableLayoutManager.getPercent(columnState.index);
            TableColumn columnWidget = tableWidget.getColumn(getCorrectedIndex(table,columnState.index)); 
            columnState.label = columnWidget.getText();            
            int correctedIndex = getCorrectedIndex(table,columnState.index);
            columnState.resizeable = tableWidget.getColumn(correctedIndex).getResizable();
            columStates.add(columnState);            
        }           
    }
    
    public boolean hasStateChanged(boolean[] stateValues) {
        newStateValues = stateValues;           
        // First call
        if (this.stateValues == null) {
            this.stateValues = stateValues;
            return true;
        }
        // Count of StateValues does not match 
        if (this.stateValues.length != stateValues.length) {
            this.stateValues = stateValues;             
            return true;
        }
        // Compare the StateValues
        for (int i=0;i<stateValues.length;i++) {
            if (this.stateValues[i] != stateValues[i]) {
                return true;
            }
        }
        return false;
    }
    
    public void showColumns(ColumnInfo[] activeColumnStates) {
        int[] vc = new int[activeColumnStates.length];
        for (int i=0;i<activeColumnStates.length;i++) {
            vc[i]=activeColumnStates[i].index;                      
        }
        for (ColumnInfo columnState : columStates) {    
            boolean isVisible = false;
            ColumnInfo currentColumnState=null;
            for (ColumnInfo visibleColumnState : activeColumnStates) {
                if (visibleColumnState.index == columnState.index) {
                    currentColumnState = visibleColumnState;                        
                    isVisible = true;
                    break;
                }
            }
            if (isVisible) {
                showTableColumn(tableLayoutManager,tableWidget,tableModel,columnState,currentColumnState);
            } else {
                hideTableColumn(tableLayoutManager,tableWidget,tableModel,columnState);
            }          
        }
        int[] columnsToHide = getColumnsToHide(originalColumnOrder,vc);            
        int[] newSortOrder = getNewSortOrder(vc,columnsToHide);            
        tableWidget.setColumnOrder(newSortOrder);
        stateValues = newStateValues;
        newStateValues = null;              
        tableLayoutManager.layout();      
    }
    
    private static int[] getNewSortOrder(int[] visibleSearchColumns, int[] columnsToHide) {
        int[] newSortOrder = new int[visibleSearchColumns.length + columnsToHide.length];
        int newSortOrderIndex = 0;
        for (int visible : visibleSearchColumns) {
            newSortOrder[newSortOrderIndex++] = visible;
        }
        for (int hide : columnsToHide) {
            newSortOrder[newSortOrderIndex++] = hide;
        }
        return newSortOrder;
    }

    private static int[] getColumnsToHide(int[] originalColumnOrder, int[] visibleSearchColumns) {
        int[] columnsToHide = new int[originalColumnOrder.length - visibleSearchColumns.length];
        int hideIndex = 0;
        for (int originalIndex : originalColumnOrder) {
            boolean isOriginalIndexVisible = false;
            for (int visible : visibleSearchColumns) {
                if (visible == originalIndex) {
                    isOriginalIndexVisible = true;
                }
            }
            if (!isOriginalIndexVisible) {
                columnsToHide[hideIndex++] = originalIndex;
            }
        }
        return columnsToHide;
    } 
    
    private static void showTableColumn(TableLayoutManager tableLayoutManager, Table tableWidget, ITableWMClient table, ColumnInfo defaultColumnState, ColumnInfo currentColumnState) {

        int correctedIndex = getCorrectedIndex(table,defaultColumnState.index);
        
        Integer absoluteWidthToSet = (currentColumnState.absoluteWidth != null) ? currentColumnState.absoluteWidth : defaultColumnState.absoluteWidth;
        Float percentToSet = (currentColumnState.percent != null) ? currentColumnState.percent : defaultColumnState.percent;        

        tableLayoutManager.setMinWidth(correctedIndex,defaultColumnState.minWidth);
        tableLayoutManager.setAbsolutWidth(correctedIndex,absoluteWidthToSet);
        if (percentToSet != null && percentToSet >=0) {
            tableLayoutManager.setPercent(correctedIndex,percentToSet);
        }
        tableWidget.getColumn(correctedIndex).setResizable(defaultColumnState.resizeable);
        
        // Set label
        String labelToSet = (currentColumnState.label != null) ? currentColumnState.label : defaultColumnState.label;
        TableColumn columnWidget = tableWidget.getColumn(correctedIndex);
        columnWidget.setText(labelToSet);     
    }       
    
    private static void hideTableColumn(TableLayoutManager tableLayoutManager, Table tableWidget, ITableWMClient table, ColumnInfo columnState) {
        
        int correctedIndex = getCorrectedIndex(table,columnState.index);
              
        tableLayoutManager.setMinWidth(correctedIndex,0);
        tableLayoutManager.setAbsolutWidth(correctedIndex,0);    
        tableWidget.getColumn(correctedIndex).setResizable(false);  
        //tableLayoutManager.collapseColumn(correctedIndex);   // Does not work correctly           
    }
    
    public static int getCorrectedIndex(ITableWMClient table, int columnIndex) {
        int correctedIndex = 0;
        for (int i=0; i<columnIndex; i++) {
            // problem - is not correct, because hidden fields are not in the Widget.
            // The widget is required to hide the column, therefore it is necessary to compute the correct index
            if (table.isVisible(i)) {
                correctedIndex++;
            }
        }
        return correctedIndex;
    }        
}
